import httpx
from argos.logging import logger

from argos.schemas import Task
from pydantic import BaseModel, Field
from typing import Type


class BaseExpectedValue(BaseModel):
    expected: str

    def get_converted(self):
        return self.expected


class ExpectedIntValue(BaseExpectedValue):
    def get_converted(self):
        return int(self.expected)


class ExpectedStringValue(BaseExpectedValue):
    pass


class BaseCheck:
    config: str
    expected_cls : Type[BaseExpectedValue] = None

    def response(self, passed, **kwargs):
        status = "success" if passed else "failure"
        return status, kwargs

    def __init__(self, client: httpx.AsyncClient, task: Task):
        self.client = client
        self.task = task

    @property
    def expected(self):
        return self.expected_cls(expected=self.task.expected).get_converted()


class HTTPStatusCheck(BaseCheck):
    config = "status-is"
    expected_cls = ExpectedIntValue

    async def run(self) -> dict:
        # XXX Get the method from the task
        task = self.task
        response = await self.client.request(method="get", url=task.url)
        logger.error(f"{response.status_code=}, {self.expected=}")
        return self.response(
            response.status_code == self.expected,
            expected=self.expected,
            retrieved=response.status_code
        )


class HTTPBodyContains(BaseCheck):
    config = "body-contains"
    expected_cls = ExpectedStringValue

    async def run(self) -> dict:
        response = await self.client.request(method="get", url=self.task.url)
        return self.response(
            self.expected in response.body
        )


class SSLCertificateExpiration(BaseCheck):
    config = "ssl-certificate-expiration"
    expected_cls = ExpectedStringValue

    async def run(self):
        return True


AVAILABLE_CHECKS = (HTTPStatusCheck, HTTPBodyContains, SSLCertificateExpiration)


class CheckNotFound(Exception):
    pass


def get_names(checks=AVAILABLE_CHECKS):
    return [c.config for c in checks]


def get_check_by_name(name, checks=AVAILABLE_CHECKS):
    checks_dict = {c.config: c for c in checks}
    check = checks_dict.get(name)
    if not check:
        raise CheckNotFound(name)
    return check
